from typing import Any, Dict, List
import random

import numpy as np

from sktmls.models import MLSLightGBMModel, MLSModelError


class LightGBMRandomContextModel(MLSLightGBMModel):
    """
    MLS 모델 레지스트리에 등록되는 LightGBM 기반 클래스입니다.

    주어진 `features`리스트를 이용해 prediction 후 cutoff 필터를 진행합니다.
    이후 `context`, `item_id`, `item_name`, `item_type`을 추가하여 반환하는 모델입니다.
    """

    def __init__(
        self,
        model,
        model_name: str,
        model_version: str,
        model_features: List[str],
        default_feature_values: List[Any],
        default_context_value: str,
        cutoff: float,
        item_id: str,
        item_name: str,
        item_type: str,
        context_features: List[str] = None,
        context_values: List[Any] = None,
    ):
        """
        ## Args

        - model: LightGBM으로 학습한 모델 객체
        - model_name: (str) 모델 이름
        - model_version: (str) 모델 버전
        - model_features: (list(str)) 학습에 사용된 피쳐 리스트
        - default_feature_values: (list) 피쳐 기본값 리스트
        - default_context_value: (str) 기본 컨텍스트 값
        - cutoff: (float) Cutoff 값
        - item_id: (str) Item ID
        - item_name: (str) Item 이름
        - item_type: (str) Item 타입
        - context_features: (optional) (list(str)) 컨텍스트 생성을 위한 피쳐 리스트 (기본값: None)
        - context_values: (optional) (list) 컨텍스트 값 리스트 (기본값: None)

        ## Example

        ```python
        my_model_v20200827 = LightGBMRandomContextModel(
            model=lightgbm_model,
            model_name="my_model",
            model_version="v20200827",
            model_features=["feature1", "feature2", "feature3", "feature4"],
            default_feature_values=[3.0, "Y", 10.0, "N"],
            default_context_value="default_context",
            cutoff=0.3,
            item_id="my_id",
            item_name="my_name",
            item_type="my_type",
            context_features=["feature5", "feature2"],
            context_values=["context_1", "context_2"],
        )

        result = my_model_v20200827.predict(["value_1", "value_2", "value_3", "value_4", "value_5", "value_2"])

        ```
        """
        if context_features and context_values:
            assert len(context_features) == len(context_values), "`context_features`와 `context_values`의 길이는 같아야 합니다."
        else:
            assert (
                not context_features and not context_values
            ), "`context_features`와 `context_values`는 둘 다 전달되거나 둘 다 전달되지 않아야 합니다."

        super().__init__(model, model_name, model_version, model_features + context_features)

        assert len(model_features) == len(
            default_feature_values
        ), "`model_features`와 `default_feature_values`의 길이가 다릅니다."
        assert None not in default_feature_values, "`default_feature_values`에 None이 포함되어 있습니다."
        self.default_feature_values = default_feature_values

        self.context_values = context_values
        self.default_context_value = default_context_value
        self.cutoff = cutoff
        self.item_id = item_id
        self.item_name = item_name
        self.item_type = item_type

    def predict(self, x: List[Any], **kwargs) -> Dict[str, Any]:
        if len(self.features) != len(x):
            raise MLSModelError("`x`와 `features`의 길이가 다릅니다.")

        model_x = x[: len(x) - len(self.context_values)]
        context_x = x[len(x) - len(self.context_values) :]

        model_x = [f if f is not None else self.default_feature_values[i] for i, f in enumerate(model_x)]

        y = self.models[0].predict(np.array([model_x])).flatten().tolist()[0]
        if y < self.cutoff:
            return {"items": []}

        contexts = [self.context_values[i] for i, f in enumerate(context_x) if f == "Y"]
        if len(contexts) == 0:
            context = self.default_context_value
        else:
            context = random.choice(contexts)

        return {
            "items": [
                {
                    "id": self.item_id,
                    "name": self.item_name,
                    "type": self.item_type,
                    "props": {"score": y, "context_id": context},
                }
            ]
        }



import random
from collections import OrderedDict
from datetime import datetime
from typing import Any, Dict, List

from pytz import timezone

from sktmls.models import MLSRuleModel, MLSModelError

TZ = timezone("Asia/Seoul")
SEED_FORMAT = {
    "hourly": "%Y%m%d%H",
    "daily": "%Y%m%d",
    "weekly": "%Y%W",
    "monthly": "%Y%m",
    "yearly": "%Y",
    "none": "%c%f",
}


class MbrCustomBenefitRandomModel(MLSRuleModel):
    """
    MLS 모델 레지스트리에 등록되는 Rule 기반 클래스입니다.

    사전정의된 컨텍스트 기반 추천에 특화된 모델입니다.
    """

    def __init__(
        self,
        model_name: str,
        model_version: str,
        features: List[str],
        categories: List[str],
        item_type: str,
        contexts: Dict[str, List] = None,
        num_pick: int = 1,
        random_cycle: str = "daily",
    ):
        """
        ## Args

        - model_name: (str) 모델 이름
        - model_version: (str) 모델 버전
        - features: (list(str)) 피쳐 리스트 (필수 피쳐: `user_id`)
        - categories: (list(str)) 전체 카테고리 리스트. 필수 피쳐 이후 피쳐 순서와 동일해야 합니다.
        - item_type: (str) 결과의 `type`에 표기할 값
        - contexts: (dict) context_id와 연결된 benefit 리스트가 key, value 인 딕셔너리.
        - num_pick: (optional) (int) 랜덤 추출할 갯수 (기본값: 1)
        - random_cycle: (optional) (str) 랜덤 시드 초기화 주기 (`hourly`|`daily`|`weekly`|`monthly`|`yearly`|`none`) (기본값: `daily`)
            - `none`의 경우 매 호출마다 초기화합니다.

        ## Example

        ```python
        my_model_v1 = MbrCustomBenefitRandomModel(
            model_name="my_model",
            model_version="v1",
            features=["user_id", "available_benefit_activity", "available_benefit_asianfood", "available_benefit_beauty"],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            item_type="mbr_custom_benefit",
            contexts={"37": ["97", "27", "32", "99", "26", "133", "84", "37", "15", "38", "85", "48", "47", "12"]},
            num_pick=14,
            random_cycle="daily",
        )
        result = my_model_v1.predict(["1234567890", None, ["1", "5", "6"], ["a1", "a2", "a3", "a4"]])
        ```
        """
        assert (
            isinstance(features, list) and len(features) > 0 and features[0] == "user_id"
        ), "`features`가 유효하지 않습니다. `features`의 첫 값은 반드시 `user_id`이어야 합니다."
        assert isinstance(categories, list) and len(categories) + 1 == len(
            features
        ), "전체 카테고리 리스트 길이가 필수 피쳐를 제외한 피쳐의 수와 다릅니다."
        assert (
            isinstance(contexts, dict) and len(contexts) > 0
        ), "`contexts`은 반드시 키가 문자열이고 값이 리스트인 딕셔너리 타입이고 적어도 한개의 아이템을 가져야합니다"
        assert isinstance(num_pick, int) and num_pick > 0, "`num_pick`이 유효하지 않습니다. 반드시 1 이상의 값을 가져야 합니다."
        assert random_cycle in SEED_FORMAT, "허용되지 않는 `random_cycle`값입니다."

        super().__init__(model_name, model_version, features)

        self.categories = categories
        self.item_type = item_type
        self.contexts = contexts
        self.num_pick = num_pick
        self.random_cycle = random_cycle

    def predict(self, x: List[Any], **kwargs) -> Dict[str, Any]:
        for i, f in enumerate(x[1:]):
            if f is not None and not isinstance(f, list):
                raise MLSModelError(f"MbrCustomBenefitRandomModel: {self.features[i + 1]}이 리스트 형식으로 전달되지 않았습니다.")
        user_id = x[0]

        available_benefit = {
            category: x[i + 1]
            for i, category in enumerate(self.categories)
            if isinstance(x[i + 1], list) and len(x[i + 1]) >= 1
        }

        random.seed(f"{user_id}{self.item_type}{datetime.now(TZ).strftime(SEED_FORMAT[self.random_cycle])}")
        context_id = random.choice(list(self.contexts.keys()))

        target_benefit_list = self.contexts[context_id]

        available_benefit_list = sum(available_benefit.values(), [])
        valid_target_benefit_list = list(
            OrderedDict.fromkeys(filter(lambda x: x in available_benefit_list, target_benefit_list))
        )

        candidates = [
            {
                "id": benefit,
                "name": "#",
                "type": self.item_type,
                "props": {"context_id": context_id},
            }
            for benefit in valid_target_benefit_list
        ]

        num_pick = min(self.num_pick, len(candidates))
        top_candidates = candidates[:num_pick]

        return {"items": top_candidates}



import random
from datetime import datetime
from typing import Any, Dict, List

from pytz import timezone

from sktmls.models import MLSRuleModel, MLSModelError

TZ = timezone("Asia/Seoul")
SEED_FORMAT = {"hourly": "%Y%m%d%H", "daily": "%Y%m%d", "weekly": "%Y%W", "monthly": "%Y%m", "yearly": "%Y"}


class MbrBoostparkBaselineModel(MLSRuleModel):
    """
    MLS 모델 레지스트리에 등록되는 Rule 기반 클래스입니다.

    멤버십 Boostpark 추천에 특화된 모델입니다.
    """

    def __init__(
        self,
        model_name: str,
        model_version: str,
        features: List[str],
        item_type: str,
        num_pick: int = 1,
        random_cycle: str = "daily",
    ):
        """
        ## Args

        - model_name: (str) 모델 이름
        - model_version: (str) 모델 버전
        - features: (list(str)) 피쳐 리스트 (필수 피쳐: `user_id`, `available_boostpark_location`)
        - item_type: (str) 결과의 `type`에 표기할 값
        - num_pick: (optional) (int) 랜덤 추출할 갯수 (기본값: 1)
        - random_cycle: (optional) (str) 랜덤 시드 초기화 주기 (`hourly`|`daily`|`weekly`|`monthly`|`yearly`) (기본값: `daily`)

        ## Example

        ```python
        my_model_v1 = MbrBoostparkBaselineModel(
            model_name="my_model",
            model_version="v1",
            features=["user_id", "available_boostpark_location"],
            item_type="mbr_boostpark",
            num_pick=3,
            random_cycle="daily"
        )

        result = my_model_v1.predict(["1234567890", ["2234", "2308", "2306", "2286"], 37])
        ```
        """
        assert (
            isinstance(features, list) and len(features) > 0 and features[0] == "user_id"
        ), "`features`가 유효하지 않습니다. `features`의 첫 값은 반드시 `user_id`이어야 합니다."
        assert isinstance(num_pick, int) and num_pick > 0, "`num_pick`이 유효하지 않습니다. 반드시 1 이상의 값을 가져야 합니다."
        assert random_cycle in SEED_FORMAT, "허용되지 않는 `random_cycle`값입니다."

        super().__init__(model_name, model_version, features)

        self.item_type = item_type
        self.num_pick = num_pick
        self.random_cycle = random_cycle

    def predict(self, x: List[Any], **kwargs) -> Dict[str, Any]:
        if len(x) != 2:
            raise MLSModelError("MbrBoostparkBaselineModel: x의 길이가 2이어야 합니다.")

        user_id = x[0]
        available_boostpark_location = x[1]

        random.seed(f"{user_id}{self.item_type}{datetime.now(TZ).strftime(SEED_FORMAT[self.random_cycle])}")

        if available_boostpark_location is None:
            return {"items": []}

        if not isinstance(available_boostpark_location, dict):
            raise MLSModelError("MbrBoostparkBaselineModel: `available_boostpark_location`이 유효하지 않습니다. dict 형식이어야 합니다.")

        available_benefit = available_boostpark_location.get("available_benefit")
        boostpark = available_boostpark_location.get("boostpark")
        context = available_boostpark_location.get("context")

        if not available_benefit or not isinstance(available_benefit, list):
            raise MLSModelError("MbrBoostparkBaselineModel: `available_benefit`이 유효하지 않습니다. 리스트 형식이어야 합니다.")
        if not boostpark:
            raise MLSModelError("MbrBoostparkBaselineModel: `boostpark`가 유효하지 않습니다.")
        if not context:
            raise MLSModelError("MbrBoostparkBaselineModel: `context`가 유효하지 않습니다.")

        candidates = [
            {
                "id": i,
                "name": "#",
                "type": self.item_type,
                "props": {
                    "boostpark": boostpark,
                    "score": str(round(random.random(), 4)),
                    "location_context": context,
                },
            }
            for i in available_benefit
        ]
        num_pick = min(self.num_pick, len(available_benefit))
        top_candidates = sorted(candidates, key=lambda k: float(k["props"]["score"]), reverse=True)[:num_pick]

        return {"items": top_candidates}



import calendar
import hashlib
from datetime import datetime
from typing import Any, Dict, List

import numpy as np
from pytz import timezone

from sktmls import MLSRuntimeENV
from sktmls.apis import MLSProfileAPIClient
from sktmls.models import MLSRuleModel

TZ = timezone("Asia/Seoul")
SEED_FORMAT = {
    "hourly": "%Y%m%d%H",
    "daily": "%Y%m%d",
    "weekly": "%Y%W",
    "monthly": "%Y%m",
    "yearly": "%Y",
    "none": "%c%f",
}


class MABRankingRuleModel(MLSRuleModel):
    """
    MLS 모델 레지스트리에 등록되는 Rule 기반 클래스입니다.

    Thompson Sampling 기반의 score를 `props` 내에 정렬하여 전달하는 모델입니다.
    """

    def __init__(
        self,
        model_name: str,
        model_version: str,
        feature: str,
        mab_item_profile_info: Dict[str, str],
        item_info: Dict[str, str],
        num_pick: int = 0,
        random_cycle: str = "daily",
        tday: str = None,
    ):
        """
        ## Args

        - model_name: (str) 모델 이름
        - model_version: (str) 모델 버전
        - feature: (str) 아이템 프로파일 테이블의 `item_id`를 전달 받을 feature 이름
        - mab_item_profile_info: (dict) MAB Thompson Sampling 파라미터가 저장된 아이템 프로파일 정보 (아래 예시 참조)
            - profile_id: (str) 조회할 아이템 프로파일 profile_id
            - key: (str) MAB 파라미터를 담고 있는 아이템 프로파일 key
        - item_info: (dict) 결과에 표기할 정보 (아래 예시 참조)
            - id: (str) 결과의 `id`에 표기할 값
            - name: (str) 결과의 `name`에 표기할 값
            - type: (str) 결과의 `type`에 표기할 값
            - props_key: (str) 랭킹을 전달할 `props` 내 키 값
        - num_pick: (optional) (int) 상위 score 추출 개수 (0인 경우 전체) (기본값: 0)
        - random_cycle: (optional) (str) 랜덤 시드 초기화 주기 (`hourly`|`daily`|`weekly`|`monthly`|`yearly`|`none`) (기본값: `daily`)
            - `none`의 경우 매 호출마다 초기화합니다.
        - tday: (optional) (str) T데이에 priority를 가지는 ranking candidate의 이름 (None이거나 빈값인 경우 T데이 미고려) (기본값: None)

        ## Example

        ```python
        my_model_v1 = MABRankingRuleModel(
            model_name="my_model",
            model_version="v1",
            feature="seg_id",
            mab_item_profile_info={
                "profile_id": "my_profile",
                "key": "card_ranking",
            },
            item_info={
                "id": "mp_point_card_ranking_seg",
                "name": "포인트탭카드랭킹",
                "type": "mp_point_card_ranking_seg",
                "props_key": "ranking",
            },
            num_pick=0,
            random_cycle="daily",
            tday="mp_point_tday_seg",
        )

        result = my_model_v1.predict(["seg_id"])
        ```
        """
        assert isinstance(feature, str) and len(feature) > 0, "`feature`는 유효한 문자열이어야 합니다."
        assert isinstance(mab_item_profile_info, dict), "`mab_item_profile_info`는 dict 타입이어야 합니다."
        assert "profile_id" in mab_item_profile_info, "`mab_item_profile_info`에 `profile_id` 값이 없습니다."
        assert "key" in mab_item_profile_info, "`mab_item_profile_info`에 `key` 값이 없습니다."
        assert isinstance(item_info, dict), "`item_info`는 dict 타입이어야 합니다."
        assert "id" in item_info, "`item_info`에 `id` 값이 없습니다."
        assert "name" in item_info, "`item_info`에 `name` 값이 없습니다."
        assert "type" in item_info, "`item_info`에 `type` 값이 없습니다."
        assert "props_key" in item_info, "`item_info`에 `props_key` 값이 없습니다."
        assert isinstance(num_pick, int) and num_pick >= 0, "`num_pick`이 유효하지 않습니다. 반드시 0 또는 양의 정수를 가져야 합니다."
        assert random_cycle in SEED_FORMAT, "허용되지 않는 `random_cycle`값입니다."
        assert tday is None or isinstance(tday, str) and len(tday) > 0, "`tday`는 None 또는 유효한 문자열이어야 합니다."

        super().__init__(model_name, model_version, [feature])

        self.mab_item_profile_info = mab_item_profile_info
        self.item_info = item_info
        self.num_pick = num_pick
        self.random_cycle = random_cycle
        self.tday = tday

    def predict(self, x: List[Any], **kwargs) -> Dict[str, Any]:
        pf_client = kwargs.get("pf_client") or MLSProfileAPIClient(runtime_env=MLSRuntimeENV.MMS)

        profile_id = self.mab_item_profile_info["profile_id"]
        item_id = x[0]
        key = self.mab_item_profile_info["key"]

        item_profile = pf_client.get_item_profile(profile_id=profile_id, item_id=item_id, keys=[key])
        mab_params = item_profile[key]

        now = datetime.now(TZ)
        seed = f"{item_id}{now.strftime(SEED_FORMAT[self.random_cycle])}"
        np.random.seed(int(hashlib.sha256(seed.encode()).hexdigest(), 16) % 10000)

        scores = {}
        for k, v in mab_params.items():
            try:
                scores[k] = np.random.beta(v[0], v[1])
            except (IndexError, TypeError):
                scores[k] = np.random.beta(1, 1)

        if self.tday:
            weeks = calendar.monthcalendar(now.year, now.month)
            first_full_week = weeks[0] if weeks[0][0] == 1 else weeks[1]
            if now.weekday() == 2 or now.day in first_full_week:
                scores[self.tday] = 1.0

        scores = sorted(list(scores.items()), key=lambda x: x[1], reverse=True)
        num_pick = min(self.num_pick, len(scores)) if self.num_pick > 0 else len(scores)

        return {
            "items": [
                {
                    "id": self.item_info["id"],
                    "name": self.item_info["name"],
                    "type": self.item_info["type"],
                    "props": {self.item_info["props_key"]: [k[0] for k in scores[:num_pick]]},
                }
            ]
        }



from typing import Dict, Any, List, Union

from dateutil import parser

from sktmls import MLSClient, MLSENV, MLSRuntimeENV, MLSResponse, MLSClientError
from sktmls.models import ManualModel, AutoMLModel, MLModelClient
from sktmls.filters import Filter
from sktmls.channels import Channel

MLS_EXPERIMENTS_API_URL = "/api/v1/experiments"


class Experiment:
    """
    MLS 실험 클래스입니다.
    """

    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) 실험 고유 ID
            - channels: (list(`sktmls.channels.Channel`)) 실험이 속한 채널 리스트
            - name: (str) 실험 이름
            - description: (str) 실험 설명
            - model_filter: (`sktmls.filters.Filter`) 적용된 필터 (필터 조건 제외)
            - bucketing_seed: (str) 버킷팅 시드
            - user: (str) 실험 소유 계정명
            - created_at: (datetime) 생성일시
            - updated_at: (datetime) 수정일시

        ## Returns
        `sktmls.experiments.Experiment`
        """
        self.id = kwargs.get("id")
        self.channels = kwargs.get("channels")
        self.channels = (
            [Channel(**channel) for channel in kwargs.get("channels")]
            if kwargs.get("channels") is not None
            else kwargs.get("channels")
        )
        self.name = kwargs.get("name")
        self.description = kwargs.get("description")
        self.user = kwargs.get("user")
        self.model_filter = (
            Filter(**kwargs.get("model_filter"))
            if kwargs.get("model_filter") is not None
            else kwargs.get("model_filter")
        )
        self.bucketing_seed = kwargs.get("bucketing_seed")

        try:
            self.created_at = parser.parse(kwargs.get("created_at"))
        except TypeError:
            self.created_at = None

        try:
            self.updated_at = parser.parse(kwargs.get("updated_at"))
        except TypeError:
            self.updated_at = None

    def get(self):
        return self.__dict__

    def reset(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def generate_request_body(self) -> Dict:
        body = {
            "name": self.name,
            "bucketing_seed": self.bucketing_seed,
            "model_filter_id": self.model_filter.id if self.model_filter is not None else None,
        }
        if self.description:
            body["description"] = self.description
        return body


class Bucket:
    """
    MLS 버킷 클래스입니다.
    """

    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) 버킷 고유 ID
            - experiment: (int) 버킷이 속한 실험정보
            - prediction: (`sktmls.experiments.Prediction`) 해당 버킷의 Prediction 정보
            - name: (str) 버킷 이름
            - description: (str) 버킷 설명
            - bucket_range: (str) 해당 버킷의 범위
            - created_at: (datetime) 생성일시
            - updated_at: (datetime) 수정일시

        ## Returns
        `sktmls.experiments.Bucket`

        """
        self.id = kwargs.get("id")
        self.experiment = kwargs.get("experiment")
        self.prediction = Prediction(**kwargs.get("prediction")) if kwargs.get("prediction") else None
        self.name = kwargs.get("name")
        self.description = kwargs.get("description")
        self.bucket_range = kwargs.get("bucket_range")

        try:
            self.created_at = parser.parse(kwargs.get("created_at"))
        except TypeError:
            self.created_at = None

        try:
            self.updated_at = parser.parse(kwargs.get("updated_at"))
        except TypeError:
            self.updated_at = None

    def get(self):
        return self.__dict__

    def reset(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def generate_request_body(self) -> Dict:
        data = {
            "name": self.name,
        }

        if self.description:
            data["description"] = self.description

        model_type = self.prediction.request["type"]
        data["prediction_type"] = model_type
        if model_type == "online":
            data["online_model_id"] = self.prediction.ml_model.id
        elif model_type == "batch":
            data["batch_model_id"] = self.prediction.ml_model.id
        elif model_type == "static":
            data["static_value"] = {"items": self.prediction.request["items"]}

        return data


class Prediction:
    def __init__(self, **kwargs):
        self.id = kwargs.get("id")
        self.request = kwargs.get("request")
        self.ml_model = kwargs.get("ml_model")

    def get(self):
        return self.__dict__


class ExperimentClient(MLSClient):
    """
    MLS 실험 관련 기능들을 제공하는 클라이언트입니다.
    """

    def __init__(
        self,
        env: MLSENV = None,
        runtime_env: MLSRuntimeENV = None,
        username: str = None,
        password: str = None,
    ):
        """
        ## Args

        - env: (`sktmls.MLSENV`) 접근할 MLS 환경 (`sktmls.MLSENV.DEV`|`sktmls.MLSENV.STG`|`sktmls.MLSENV.PRD`) (기본값: `sktmls.MLSENV.STG`)
        - runtime_env: (`sktmls.MLSRuntimeENV`) 클라이언트가 실행되는 환경 (`sktmls.MLSRuntimeENV.YE`|`sktmls.MLSRuntimeENV.EDD`|`sktmls.MLSRuntimeENV.LOCAL`) (기본값: `sktmls.MLSRuntimeENV.LOCAL`)
        - username: (str) MLS 계정명 (기본값: $MLS_USERNAME)
        - password: (str) MLS 계정 비밀번호 (기본값: $MLS_PASSWORD)

        아래의 환경 변수가 정의된 경우 해당 파라미터를 생략 가능합니다.

        - $MLS_ENV: env
        - $MLS_RUNTIME_ENV: runtime_env
        - $MLS_USERNAME: username
        - $MLS_PASSWORD: password

        ## Returns
        `sktmls.experiments.ExperimentClient`

        ## Example

        ```python
        experiment_client = ExperimentClient(env=MLSENV.STG, runtime_env=MLSRuntimeENV.YE, username="mls_account", password="mls_password")
        ```
        """
        super().__init__(env=env, runtime_env=runtime_env, username=username, password=password)

    def create_experiment(
        self, name: str, description: str = None, bucketing_seed: str = "", model_filter: Filter = None
    ) -> Experiment:
        """
        실험을 생성합니다.

        ## Args

        - name: (str) 생성할 실험 이름
        - description: (optional) (str) 생성할 실험 설명
        - bucketing_seed: (optional) (str) 생성할 실험의 버킷팅 시드
        - model_filter: (optional) (`sktmls.filters.Filter`) 생성할 실험의 필터 (필터 조건 제외)

        ## Returns
        `sktmls.experiments.Experiment`

        - id: (int) 실험 고유 ID
        - channels: (list(`sktmls.channels.Channel`)) 실험이 속한 채널 리스트
        - name: (str) 실험 이름
        - description: (str) 실험 설명
        - model_filter: (`sktmls.filters.Filter`) 적용된 필터 (필터 조건 제외)
        - bucketing_seed: (str) 버킷팅 시드
        - user: (str) 실험 소유 계정명
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        my_experiment = experiment_client.create_experiment(
            name="my_experiment",
            description="This is my experiment",
            filter=my_filter,
        )
        ```
        """

        data = {"name": name, "bucketing_seed": bucketing_seed}
        if description:
            data["description"] = description
        if model_filter:
            data["model_filter_id"] = model_filter.id

        return Experiment(**self._request(method="POST", url=MLS_EXPERIMENTS_API_URL, data=data).results)

    def remove_filter(self, experiment: Experiment) -> Experiment:
        """
        실험의 필터를 제거합니다.

        ## Args

        - experiment: (`sktmls.experiments.Experiment`) 필터를 제거할 실험

        ## Returns
        `sktmls.experiments.Experiment`

        - id: (int) 실험 고유 ID
        - channels: (list(`sktmls.channels.Channel`)) 실험이 속한 채널 리스트
        - name: (str) 실험 이름
        - description: (str) 실험 설명
        - model_filter: (`sktmls.filters.Filter`) 적용된 필터 (필터 조건 제외)
        - bucketing_seed: (str) 버킷팅 시드
        - user: (str) 실험 소유 계정명
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        my_experiment = experiment_client.remove_filter(
            experiment=my_experiment,
        )
        ```
        """
        assert type(experiment) == Experiment
        data = experiment.generate_request_body()
        data.pop("model_filter_id")

        self._request(method="PUT", url=f"{MLS_EXPERIMENTS_API_URL}/{experiment.id}", data=data)

        return self.get_experiment(id=experiment.id)

    def update_experiment(
        self,
        experiment: Experiment,
        name: str = None,
        description: str = None,
        bucketing_seed: str = None,
        model_filter: Filter = None,
    ) -> Experiment:
        """
        실험을 수정합니다. Optional 필드의 경우에 값을 넣지 않을시에는 기존의 값들이 유지됩니다.

        ## Args

        - experiment: (`sktmls.experiments.Experiment`) 수정할 실험
        - name: (optional) (str) 수정할 실험 이름
        - description: (optional) (str) 수정할 실험 설명
        - bucketing_seed: (optional) (str) 수정할 실험의 버킷팅 시드
        - model_filter: (optional) (`sktmls.filters.Filter`) 수정할 실험의 필터 (필터 조건 제외)

        ## Returns
        `sktmls.experiments.Experiment`

        - id: (int) 실험 고유 ID
        - channels: (list(`sktmls.channels.Channel`)) 실험이 속한 채널 리스트
        - name: (str) 실험 이름
        - description: (str) 실험 설명
        - model_filter: (`sktmls.filters.Filter`) 적용된 필터 (필터 조건 제외)
        - bucketing_seed: (str) 버킷팅 시드
        - user: (str) 실험 소유 계정명
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        my_experiment = experiment_client.update_experiment(
            experiment=my_experiment,
            description="This is updated experiment",
        )
        ```
        """
        assert type(experiment) == Experiment
        data = experiment.generate_request_body()

        if name:
            data["name"] = name
        if description:
            data["description"] = description
        if bucketing_seed:
            data["bucketing_seed"] = bucketing_seed
        if model_filter:
            data["model_filter_id"] = model_filter.id

        self._request(method="PUT", url=f"{MLS_EXPERIMENTS_API_URL}/{experiment.id}", data=data)

        return self.get_experiment(id=experiment.id)

    def get_experiment(self, id: int = None, name: str = None) -> Experiment:
        """
        실험 정보를 조회합니다.

        ## Args: `id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - id: (int) 실험 고유 ID
        - name: (str) 실험 이름

        ## Returns
        `sktmls.experiments.Experiment`

        - id: (int) 실험 고유 ID
        - channels: (list(`sktmls.channels.Channel`)) 실험이 속한 채널 리스트
        - name: (str) 실험 이름
        - description: (str) 실험 설명
        - model_filter: (`sktmls.filters.Filter`) 적용된 필터 (필터 조건 제외)
        - bucketing_seed: (str) 버킷팅 시드
        - user: (str) 실험 소유 계정명
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        experiment_by_id = experiment_client.get_experiment(id=3)
        experiment_by_name = experiment_client.get_experiment(name="my_experiment")
        ```
        """
        assert id or name, "`id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        experiments = self.list_experiments(id=id, name=name)
        if len(experiments) == 0:
            raise MLSClientError(code=404, msg="해당 실험이 없습니다")
        elif len(experiments) > 1:
            raise MLSClientError(code=409, msg="같은 이름의 실험이 여러개 존재합니다.")

        return experiments[0]

    def list_experiments(self, **kwargs) -> List[Experiment]:
        """
        실험의 리스트를 가져옵니다.

        ## Args

        - kwargs: (optional) (dict) 쿼리 조건
            - id: (int) 실험 고유 ID
            - name: (str) 실험 이름
            - query: (str) 검색 문자
            - page: (int) 페이지 번호

        ## Returns

        list(`sktmls.experiments.Experiment`)
        - id: (int) 실험 고유 ID
        - channels: (list(`sktmls.channels.Channel`)) 실험이 속한 채널 리스트
        - name: (str) 실험 이름
        - description: (str) 실험 설명
        - model_filter: (`sktmls.filters.Filter`) 적용된 필터 (필터 조건 제외)
        - bucketing_seed: (str) 버킷팅 시드
        - user: (str) 실험 소유 계정명
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        experiments = experiment_client.list_experiments()
        ```
        """

        response = self._request(method="GET", url=f"{MLS_EXPERIMENTS_API_URL}", params=kwargs).results

        return [Experiment(**experiment) for experiment in response]

    def delete_experiment(self, experiment: Experiment) -> MLSResponse:
        """
        실험을 삭제합니다.

        ## Args

        - experiment: (`sktmls.experiments.Experiment`) experiment 객체

        ## Returns
        `sktmls.MLSResponse`

        ## Example

        ```python
        experiment_client.delete_experiment(experiment)
        ```
        """
        assert type(experiment) == Experiment

        return self._request(method="DELETE", url=f"{MLS_EXPERIMENTS_API_URL}/{experiment.id}")

    def update_bucket_ratio(self, experiment: Experiment, bucket_ratio: Dict[Bucket, int]) -> List[Bucket]:
        """
        현재 실험의 버킷 비율을 수정합니다.

        ## Args

        - experiment: (`sktmls.experiments.Experiment`) 수정할 실험
        - bucket_ratio: (dict(Bucket, int)) 키가 버킷 이고 바꾸고자 하는 bucket range 가 값인 Dict

        ## Returns

        list(sktmls.experiments.Bucket)
        - id: (int) 버킷 고유 ID
        - experiment: (int) 버킷이 속한 실험정보
        - prediction: (`sktmls.experiments.Prediction`) 해당 버킷의 Prediction 정보
        - name: (str) 버킷 이름
        - description: (str) 버킷 설명
        - bucket_range: (str) 해당 버킷의 범위
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        buckets = experiment_client.list_buckets(experiment)
        bucket_ratio = {buckets[0]: 40, buckets[1]: 60}
        experiment_client.update_bucket_ratio(experiment, bucket_ratio)
        ```
        """
        assert type(experiment) == Experiment

        ratio_data = {}
        for bucket, value in bucket_ratio.items():
            assert type(bucket) == Bucket
            assert type(value) == int
            ratio_data[bucket.id] = value

        data = {"bucket_ratio": ratio_data}

        response = self._request(
            method="PUT",
            url=f"{MLS_EXPERIMENTS_API_URL}/{experiment.id}/bucket_ratio",
            data=data,
        ).results

        return [Bucket(**bucket) for bucket in response]

    def create_bucket(
        self,
        experiment: Experiment,
        name: str,
        prediction_type: str,
        model: Union[ManualModel, AutoMLModel] = None,
        description: str = None,
        static_value: Dict[str, Any] = None,
    ) -> Bucket:
        """
        버킷을 생성합니다.

        ## Args

        - experiment: (`sktmls.experiments.Experiment`) 수정할 실험
        - name: (str) 생성할 버킷 이름
        - prediction_type: 버킷의 추론 타입 (`empty`|`static`|`batch`|`online`)
        - model: (optional) (`sktmls.models.ManualModel`, `sktmls.models.AutoMLModel`) 버킷에 연결할 모델
        - static_value: (optional) (dict) (고정값일 경우) Static Value
        - description: (optional) (str) 생성할 버킷 설명

        ## Returns
        `sktmls.experiments.Bucket`

        - id: (int) 버킷 고유 ID
        - experiment: (int) 버킷이 속한 실험정보
        - prediction: (`sktmls.experiments.Prediction`) 해당 버킷의 Prediction 정보
        - name: (str) 버킷 이름
        - description: (str) 버킷 설명
        - bucket_range: (str) 해당 버킷의 범위
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        my_batch_model = model_client.get_manual_model(
            name="my_manual_model",
            version="my_manual_version",
        )
        experiment_client.create_bucket(
            experiment=my_experiment,
            name="my_bucket",
            prediction_type="batch",
            model=my_batch_model
        )
        ```
        """
        assert type(experiment) == Experiment
        assert prediction_type in ["empty", "static", "batch", "online"]

        data = {"name": name, "prediction_type": prediction_type}
        if prediction_type == "online":
            assert type(model) in [ManualModel, AutoMLModel]
            data["online_model_id"] = model.id
        elif prediction_type == "batch":
            assert type(model) in [ManualModel, AutoMLModel]
            data["batch_model_id"] = model.id
        elif prediction_type == "static":
            assert static_value
            data["static_value"] = static_value

        if description:
            data["description"] = description

        resp = self._request(method="POST", url=f"{MLS_EXPERIMENTS_API_URL}/{experiment.id}/buckets", data=data).results
        resp["experiment"] = experiment

        return Bucket(**resp)

    def list_buckets(self, experiment: Experiment, **kwargs) -> List[Bucket]:
        """
        특정 실험에 속한 버킷들의 정보를 조회합니다.

        ## Args

        - experiment: (sktmls.experiments.Experiment) 버킷의 실험 객체
        - kwargs: (optional) (dict) 쿼리 조건
            - id: (int) 버킷 고유 ID
            - name: (str) 버킷 이름
            - query: (str) 검색 문자
            - page: (int) 페이지 번호

        ## Returns
        list(`sktmls.experiments.Bucket`)

        - id: (int) 버킷 고유 ID
        - experiment: (int) 버킷이 속한 실험정보
        - prediction: (`sktmls.experiments.Prediction`) 해당 버킷의 Prediction 정보
        - name: (str) 버킷 이름
        - description: (str) 버킷 설명
        - bucket_range: (str) 해당 버킷의 범위
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        buckets = experiment_client.list_buckets(experiment)
        ```
        """
        assert type(experiment) == Experiment

        resp = self._request(
            method="GET", url=f"{MLS_EXPERIMENTS_API_URL}/{experiment.id}/buckets", params=kwargs
        ).results

        ml_model_client = MLModelClient(
            env=self._MLSClient__env,
            runtime_env=self._MLSClient__runtime_env,
            username=self._MLSClient__username,
            password=self._MLSClient__password,
        )

        buckets = []
        for r in resp:
            r["experiment"] = experiment

            model_id = r.get("prediction", {}).get("ml_model")
            model_type = r.get("prediction", {}).get("request", {}).get("type")

            if model_id:
                if model_type in ["batch", "online"]:
                    model = ml_model_client.get_manual_model(id=model_id)
                elif model_type == "automl":
                    model = ml_model_client.get_automl_model(id=model_id)
                else:
                    raise Exception(f"지원하지 않는 모델 타입입니다: {model_type}")
                r["prediction"]["ml_model"] = model
            buckets.append(Bucket(**r))

        return buckets

    def get_bucket(self, experiment: Experiment, id: int = None, name: str = None) -> Bucket:
        """
        버킷 정보를 가져옵니다.

        ## Args: `id` 또는 `name`중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - experiment: (sktmls.experiments.Experiment) 버킷의 실험 객체
        - id: (optional) (int) 버킷 고유 ID
        - name: (optional) (str) 버킷 이름

        ## Returns
        `sktmls.experiments.Bucket`

        - id: (int) 버킷 고유 ID
        - experiment: (int) 버킷이 속한 실험정보
        - prediction: (`sktmls.experiments.Prediction`) 해당 버킷의 Prediction 정보
        - name: (str) 버킷 이름
        - description: (str) 버킷 설명
        - bucket_range: (str) 해당 버킷의 범위
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        my_bucket_by_id = experiment_client.get_bucket(id=3)
        my_bucket_by_name = experiment_client.get_bucket(name="my_bucket")
        ```
        """
        assert type(experiment) == Experiment
        assert id or name, "One of id or name must be provided"

        buckets = self.list_buckets(experiment, id=id, name=name)
        if len(buckets) == 0:
            raise MLSClientError(code=404, msg="해당 버킷이 없습니다")
        elif len(buckets) > 1:
            raise MLSClientError(code=409, msg="같은 이름의 버킷이 여러개 존재합니다.")

        return buckets[0]

    def delete_bucket(self, bucket: Bucket) -> MLSResponse:
        """
        버킷을 삭제합니다.

        ## Args

        - bucket: (sktmls.experiments.Bucket) 삭제할 버킷

        ## Return
        `sktmls.MLSResponse`

        ## Example

        ```python
        bucket_to_delete = experiment_client.get_bucket(id=3)
        experiment_client.delete_bucket(bucket_to_delete)
        ```
        """
        assert type(bucket) == Bucket

        return self._request(
            method="DELETE", url=f"{MLS_EXPERIMENTS_API_URL}/{bucket.experiment.id}/buckets/{bucket.id}"
        )

    def update_bucket(
        self,
        bucket: Bucket,
        name: str = None,
        prediction_type: str = None,
        model: Union[ManualModel, AutoMLModel] = None,
        description: str = None,
        static_value: Dict[str, Any] = None,
    ) -> Bucket:
        """
        버킷을 수정합니다.

        ## Args

        - experiment: (`sktmls.experiments.Experiment`) 수정할 실험
        - name: (optional) (str) 수정할 버킷 이름
        - prediction_type: (optional) 수정할 버킷의 추론 타입 (`empty`|`static`|`batch`|`online`)
        - model: (optional) (`sktmls.models.ManualModel`, `sktmls.models.AutoMLModel`) 수정할 버킷에 연결할 모델
        - static_value: (optional) (dict) (고정값일 경우) Static Value
        - description: (optional) (str) 수정할 버킷 설명


        ## Returns
        `sktmls.experiments.Bucket`

        - id: (int) 버킷 고유 ID
        - experiment: (int) 버킷이 속한 실험정보
        - prediction: (`sktmls.experiments.Prediction`) 해당 버킷의 Prediction 정보
        - name: (str) 버킷 이름
        - description: (str) 버킷 설명
        - bucket_range: (str) 해당 버킷의 범위
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        my_batch_model = model_client.get_manual_model(
            name="my_manual_model",
            version="my_manual_version",
        )
        my_bucket = experiment_client.get_bucket(name="my_bucket")
        experiment_client.update_bucket(
            bucket=my_bucket,
            prediction_type="batch",
            model=my_batch_model
        )
        ```
        """
        assert type(bucket) == Bucket

        data = bucket.generate_request_body()
        if name:
            data["name"] = name
        if description:
            data["description"] = description
        if prediction_type:
            assert prediction_type in ["empty", "static", "batch", "online", "automl"]
            data["prediction_type"] = prediction_type
            if prediction_type == "online":
                assert type(model) in [ManualModel, AutoMLModel]
                data["online_model_id"] = model.id
            elif prediction_type == "batch":
                assert type(model) in [ManualModel, AutoMLModel]
                data["batch_model_id"] = model.id
            elif prediction_type == "static":
                assert static_value
                data["static_value"] = static_value
            elif prediction_type == "automl":
                data["automl_model_id"] = model.id

        resp = self._request(
            method="PUT", url=f"{MLS_EXPERIMENTS_API_URL}/{bucket.experiment.id}/buckets/{bucket.id}", data=data
        ).results
        resp["experiment"] = bucket.experiment
        resp["prediction"]["ml_model"] = model

        bucket.reset(**resp)

        return bucket



import random
from datetime import datetime
from typing import Any, Dict, List

from pytz import timezone

from sktmls.models import MLSRuleModel, MLSModelError

TZ = timezone("Asia/Seoul")
SEED_FORMAT = {"hourly": "%Y%m%d%H", "daily": "%Y%m%d", "weekly": "%Y%W", "monthly": "%Y%m", "yearly": "%Y"}


class RandomScoreModel(MLSRuleModel):
    """
    MLS 모델 레지스트리에 등록되는 Rule 기반 클래스입니다.

    고정된 추천 상품에 대해 random score를 반환합니다.
    """

    def __init__(self, model_name: str, model_version: str, item_id: str, item_name: str, item_type: str):
        """
        ## Args

        - model_name: (str) 모델 이름
        - model_version: (str) 모델 버전
        - item_id: (str) 결과의 `id`에 표기할 값
        - item_name: (str) 결과의 `name`에 표기할 값
        - item_type: (str) 결과의 `type`에 표기할 값

        ## Example

        ```python
        my_model_v1 = RandomScoreModel(
            model_name="my_model",
            model_version="v1",
            item_id="item1",
            item_name="아이템1",
            item_type="type"
        )

        result = my_model_v1.predict(None)
        ```
        """
        super().__init__(model_name, model_version, ["user_id"])

        self.item_id = item_id
        self.item_name = item_name
        self.item_type = item_type

    def predict(self, x: List[Any], **kwargs) -> Dict[str, Any]:
        return {
            "items": [
                {
                    "id": self.item_id,
                    "name": self.item_name,
                    "type": self.item_type,
                    "props": {"score": str(round(random.random(), 4))},
                }
            ]
        }


class PeriodicRandomScoreModel(RandomScoreModel):
    """
    MLS 모델 레지스트리에 등록되는 Rule 기반 클래스입니다.

    고정된 추천 상품에 대해 random score를 반환합니다.

    `random_cycle` 값을 참조하여 랜덤 시드를 설정합니다.
    """

    def __init__(
        self,
        model_name: str,
        model_version: str,
        item_id: str,
        item_name: str,
        item_type: str,
        random_cycle: str = "daily",
    ):
        """
        ## Args

        - model_name: (str) 모델 이름
        - model_version: (str) 모델 버전
        - item_id: (str) 결과의 `id`에 표기할 값
        - item_name: (str) 결과의 `name`에 표기할 값
        - item_type: (str) 결과의 `type`에 표기할 값
        - random_cycle: (optional) (str) 랜덤 시드 초기화 주기 (`hourly`|`daily`|`weekly`|`monthly`|`yearly`) (기본값: `daily`)

        ## Example

        ```python
        my_model_v1 = PeriodicRandomScoreModel(
            model_name="my_model",
            model_version="v1",
            item_id="item1",
            item_name="아이템1",
            item_type="type",
            random_cycle="weekly"
        )

        result = my_model_v1.predict("my_svc_mgmt_num")
        ```
        """
        super().__init__(model_name, model_version, item_id, item_name, item_type)

        if random_cycle not in SEED_FORMAT:
            raise MLSModelError("허용되지 않는 `random_cycle`값입니다.")
        self.random_cycle = random_cycle

    def predict(self, x: List[Any], **kwargs) -> Dict[str, Any]:
        user_id = x[0]
        random.seed(f"{user_id}{self.item_type}{datetime.now(TZ).strftime(SEED_FORMAT[self.random_cycle])}")
        return super().predict(x)



import os
from typing import Any, Dict, List

import requests

from sktmls import MLSENV, MLSRuntimeENV, MLSResponse
from sktmls.config import Config


class MLSProfileAPIClient:
    """
    MLS 프로파일 API를 호출할 수 있는 클라이언트입니다.

    MMS를 제외한 환경에서는 테스트용 호출로 처리됩니다.

    EDD 환경은 지원하지 않습니다.
    """

    def __init__(
        self,
        env: MLSENV = None,
        runtime_env: MLSRuntimeENV = None,
        client_id: str = None,
        apikey: str = None,
    ):
        """
        ## Args

        - env: (`sktmls.MLSENV`) 접근할 MLS 환경 (`sktmls.MLSENV.DEV`|`sktmls.MLSENV.STG`|`sktmls.MLSENV.PRD`) (기본값: `sktmls.MLSENV.STG`)
        - runtime_env: (`sktmls.MLSRuntimeENV`) 클라이언트가 실행되는 환경 (`sktmls.MLSRuntimeENV.YE`|`sktmls.MLSRuntimeENV.MMS`|`sktmls.MLSRuntimeENV.LOCAL`) (기본값: `sktmls.MLSRuntimeENV.LOCAL`)
        - client_id: (optional) (str) 호출 클라이언트 (`tw`, `twd`, `netcrm` 등)
        - apikey: (optional) (str) 클라이언트의 API Key

        아래의 환경 변수가 정의된 경우 해당 파라미터를 생략 가능합니다.

        - $MLS_ENV: env
        - $AWS_ENV: env
        - $MLS_RUNTIME_ENV: runtime_env

        runtime_env가 `sktmls.MLSRuntimeENV.MMS`인 경우 `client_id`와 `apikey` 파라미터를 생략 가능합니다.

        ## Returns
        `sktmls.apis.MLSProfileAPIClient`

        ## Example

        ```python
        profile_api_client = MLSProfileAPIClient(env=MLSENV.STG, runtime_env=MLSRuntimeENV.YE)
        ```
        """
        if env:
            assert env in MLSENV.list_items(), "유효하지 않은 MLS 환경입니다."
            self.__env = env
        elif os.environ.get("MLS_ENV"):
            self.__env = MLSENV[os.environ["MLS_ENV"]]
        elif os.environ.get("AWS_ENV"):
            self.__env = MLSENV[os.environ["AWS_ENV"]]
        else:
            self.__env = MLSENV.STG

        if runtime_env:
            assert runtime_env in MLSRuntimeENV.list_items(), "유효하지 않은 런타임 환경입니다."
            self.__runtime_env = runtime_env
        elif os.environ.get("MLS_RUNTINE_ENV"):
            self.__runtime_env = MLSRuntimeENV[os.environ["MLS_RUNTINE_ENV"]]
        else:
            __HOSTNAME = os.environ.get("HOSTNAME", "").lower()
            if __HOSTNAME.startswith("bdp-dmi"):
                self.__runtime_env = MLSRuntimeENV.YE
            elif __HOSTNAME.startswith("vm-skt"):
                self.__runtime_env = MLSRuntimeENV.EDD
            else:
                self.__runtime_env = MLSRuntimeENV.LOCAL

        assert runtime_env != MLSRuntimeENV.EDD, "EDD 환경은 지원하지 않습니다."
        if runtime_env != MLSRuntimeENV.MMS:
            assert client_id and apikey, "`client_id`와 `apikey`가 전달되어야 합니다."
            self.__client_id = client_id
            self.__apikey = apikey
        else:
            self.__client_id = None
            self.__apikey = None

        self.config = Config(self.__runtime_env.value)

    def get_env(self) -> MLSENV:
        return self.__env

    def get_runtime_env(self) -> MLSRuntimeENV:
        return self.__runtime_env

    def get_user_profile(self, profile_id: str, user_id: str, keys: List[str] = None) -> Dict[str, Any]:
        """
        MLS User Profile API를 호출합니다.

        ## Args

        - profile_id: (str) 호출할 프로파일 ID
        - user_id: (str) 해시된 유저 고유 ID (해시된 서비스관리번호)
        - keys: (optional) (list(str)) 조회할 키 리스트 (기본값: None, 기본값 전달시 모든 키 조회)

        ## Returns
        dict

        ## Example

        ```python
        user_profile_dict = profile_api_client.get_user_profile(
            profile_id="default",
            user_id=hashlib.sha256("1234567890".encode()).hexdigest(),
            keys=["age", "gender"]
        )
        ```
        """
        return self._get_profile(api_type="user", profile_id=profile_id, target_id=user_id, keys=keys)

    def get_item_profile(self, profile_id: str, item_id: str, keys: List[str] = None) -> Dict[str, Any]:
        """
        MLS Item Profile API를 호출합니다.

        ## Args

        - profile_id: (str) 호출할 프로파일 ID
        - item_id: (str) 아이템 고유 ID
        - keys: (optional) (list(str)) 조회할 키 리스트 (기본값: None, 기본값 전달시 모든 키 조회)

        ## Returns
        dict

        ## Example

        ```python
        item_profile_dict = profile_api_client.get_item_profile(
            profile_id="device",
            item_id="item001",
            keys=["price", "resolution"]
        )
        ```
        """

        return self._get_profile(api_type="item", profile_id=profile_id, target_id=item_id, keys=keys)

    def _get_profile(self, api_type: str, profile_id: str, target_id: str, keys: List[str] = None) -> Dict[str, Any]:
        params = {}
        headers = {}
        if keys is not None:
            assert isinstance(keys, list), "`keys` 리스트가 유효하지 않습니다."
            params["f"] = keys

        if self.__runtime_env == MLSRuntimeENV.MMS:
            client_id = "mls-mms"
        else:
            client_id = self.__client_id
            headers["apikey"] = self.__apikey
            params["mode"] = "test"

        api_type += "s"
        response = MLSResponse(
            requests.get(
                url=f"{self.config.MLS_PROFILE_API_URL[self.__env.value]}/v1/{client_id}/profiles/{profile_id}/{api_type}/{target_id}",
                headers=headers,
                params=params,
            )
        )
        return response.results



from sktmls.models.contrib import MbrSimilarBenefitModel


class TestMbrSimilarBenefitModel:
    def test_000_mbr_valid_1a(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "brand": {
                    "brand_id": "test_brand",
                },
                "similar_items": ["a3", "b2", "c9"],
            },
        )

        mbm = MbrSimilarBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "use_benefit",
                "like_benefit",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            contexts=[{"act": "use", "context_id": 35}, {"act": "like", "context_id": 36}],
            item_type="test_type",
            num_pick=1,
        )

        results = mbm.predict(
            [
                "1234567890",
                ["use_benefit1", "use_benefit2", "use_benefit3"],
                ["like_benefit1", "like_benefit2", "like_benefit3"],
                ["a1", "a2", "a3", "a4"],
                None,
                ["b1", "b2", "3", "4"],
            ]
        )
        assert len(results["items"]) == 1

        item = results["items"][0]
        assert item["id"] == "a3"
        assert item["type"] == "test_type"
        assert item["props"]["context_id"] in [35, 36]
        if item["props"]["context_id"] == 35:
            item["props"]["benefit_id"] in ["use_benefit1", "use_benefit2", "use_benefit3"]
        else:
            item["props"]["benefit_id"] in ["like_benefit1", "like_benefit2", "like_benefit3"]

    def test_001_mbr_valid_1b(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "brand": {
                    "brand_id": "test_brand",
                },
                "similar_items": ["b2", "a3", "c9"],
            },
        )

        mbm = MbrSimilarBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "use_benefit",
                "like_benefit",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            contexts=[{"act": "use", "context_id": 35}, {"act": "like", "context_id": 36}],
            item_type="test_type",
            num_pick=1,
        )

        results = mbm.predict(
            [
                "1234567890",
                ["use_benefit1", "use_benefit2", "use_benefit3"],
                ["like_benefit1", "like_benefit2", "like_benefit3"],
                ["a1", "a2", "a3", "a4"],
                None,
                ["b1", "b2", "3", "4"],
            ]
        )
        assert len(results["items"]) == 1

        item = results["items"][0]
        assert item["id"] == "b2"
        assert item["type"] == "test_type"
        assert item["props"]["context_id"] in [35, 36]
        if item["props"]["context_id"] == 35:
            item["props"]["benefit_id"] in ["use_benefit1", "use_benefit2", "use_benefit3"]
        else:
            item["props"]["benefit_id"] in ["like_benefit1", "like_benefit2", "like_benefit3"]

    def test_002_mbr_valid_2(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "brand": {
                    "brand_id": "test_brand",
                },
                "similar_items": ["a3", "b2", "c9"],
            },
        )

        mbm = MbrSimilarBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "use_benefit",
                "like_benefit",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            contexts=[{"act": "use", "context_id": 35}, {"act": "like", "context_id": 36}],
            item_type="test_type",
            num_pick=2,
        )

        results = mbm.predict(
            [
                "1234567890",
                ["use_benefit1", "use_benefit2", "use_benefit3"],
                ["like_benefit1", "like_benefit2", "like_benefit3"],
                ["a1", "a2", "a3", "a4"],
                None,
                ["b1", "b2", "3", "4"],
            ]
        )
        assert len(results["items"]) == 2

        item = results["items"][0]
        assert item["id"] == "a3"
        assert item["type"] == "test_type"
        assert item["props"]["context_id"] in [35, 36]
        if item["props"]["context_id"] == 35:
            item["props"]["benefit_id"] in ["use_benefit1", "use_benefit2", "use_benefit3"]
        else:
            item["props"]["benefit_id"] in ["like_benefit1", "like_benefit2", "like_benefit3"]

        item = results["items"][1]
        assert item["id"] == "b2"
        assert item["type"] == "test_type"
        assert item["props"]["context_id"] in [35, 36]
        if item["props"]["context_id"] == 35:
            item["props"]["benefit_id"] in ["use_benefit1", "use_benefit2", "use_benefit3"]
        else:
            item["props"]["benefit_id"] in ["like_benefit1", "like_benefit2", "like_benefit3"]

    def test_003_mbr_valid_3(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "brand": {
                    "brand_id": "test_brand",
                },
                "similar_items": ["a3", "b2", "c9"],
            },
        )

        mbm = MbrSimilarBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "use_benefit",
                "like_benefit",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            contexts=[{"act": "use", "context_id": 35}, {"act": "like", "context_id": 36}],
            item_type="test_type",
            num_pick=3,
        )

        results = mbm.predict(
            [
                "1234567890",
                ["use_benefit1", "use_benefit2", "use_benefit3"],
                ["like_benefit1", "like_benefit2", "like_benefit3"],
                ["a1", "a2", "a3", "a4"],
                None,
                ["b1", "b2", "3", "4"],
            ]
        )
        assert len(results["items"]) == 2

        item = results["items"][0]
        assert item["id"] == "a3"
        assert item["type"] == "test_type"
        assert item["props"]["context_id"] in [35, 36]
        if item["props"]["context_id"] == 35:
            item["props"]["benefit_id"] in ["use_benefit1", "use_benefit2", "use_benefit3"]
        else:
            item["props"]["benefit_id"] in ["like_benefit1", "like_benefit2", "like_benefit3"]

        item = results["items"][1]
        assert item["id"] == "b2"
        assert item["type"] == "test_type"
        assert item["props"]["context_id"] in [35, 36]
        if item["props"]["context_id"] == 35:
            item["props"]["benefit_id"] in ["use_benefit1", "use_benefit2", "use_benefit3"]
        else:
            item["props"]["benefit_id"] in ["like_benefit1", "like_benefit2", "like_benefit3"]

    def test_004_mbr_valid_only_use(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "brand": {
                    "brand_id": "test_brand",
                },
                "similar_items": ["a3", "b2", "c9"],
            },
        )

        mbm = MbrSimilarBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "use_benefit",
                "like_benefit",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            contexts=[{"act": "use", "context_id": 35}, {"act": "like", "context_id": 36}],
            item_type="test_type",
            num_pick=2,
        )

        results = mbm.predict(
            [
                "1234567890",
                ["use_benefit1", "use_benefit2", "use_benefit3"],
                None,
                ["a1", "a2", "a3", "a4"],
                None,
                ["b1", "b2", "3", "4"],
            ]
        )
        assert len(results["items"]) == 2

        item = results["items"][0]
        assert item["id"] == "a3"
        assert item["type"] == "test_type"
        assert item["props"]["context_id"] == 35
        item["props"]["benefit_id"] in ["use_benefit1", "use_benefit2", "use_benefit3"]

        item = results["items"][1]
        assert item["id"] == "b2"
        assert item["type"] == "test_type"
        assert item["props"]["context_id"] == 35
        item["props"]["benefit_id"] in ["use_benefit1", "use_benefit2", "use_benefit3"]

    def test_005_mbr_valid_all_none(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "brand": {
                    "brand_id": "test_brand",
                },
                "similar_items": ["a3", "b2", "c9"],
            },
        )

        mbm = MbrSimilarBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "use_benefit",
                "like_benefit",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            contexts=[{"act": "use", "context_id": 35}, {"act": "like", "context_id": 36}],
            item_type="test_type",
            num_pick=2,
        )

        results = mbm.predict(
            [
                "1234567890",
                None,
                None,
                ["a1", "a2", "a3", "a4"],
                None,
                ["b1", "b2", "3", "4"],
            ]
        )
        assert len(results["items"]) == 0
